<?php

/**
 * @file
 * Contains Drupal\library\Plugin\Field\FieldWidget\LibraryItemFieldWidget.
 */

namespace Drupal\library\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'library_item_field_widget' widget.
 *
 * @FieldWidget(
 *   id = "library_item_field_widget",
 *   label = @Translation("Library item widget"),
 *   field_types = {
 *     "entity_reference"
 *   }
 * )
 */
class LibraryItemFieldWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $entity = $items->getEntity();
    $referenced_entities = $items->referencedEntities();
    $element += array(
      '#type' => 'entity_autocomplete',
      '#target_type' => $this->getFieldSetting('target_type'),
      '#selection_handler' => $this->getFieldSetting('handler'),
      // Entity reference field items are handling validation themselves via
      // the 'ValidReference' constraint.
      '#validate_reference' => TRUE,
      '#maxlength' => 1024,
      '#default_value' => isset($referenced_entities[$delta]) ? $referenced_entities[$delta] : NULL,
      '#size' => $this->getSetting('size'),
      '#placeholder' => $this->getSetting('placeholder'),
      '#autocreate' => array(
        'bundle' => 'library_item',
        'uid' => ($entity instanceof EntityOwnerInterface) ? $entity->getOwnerId() : \Drupal::currentUser()->id()
      )
    );

    return array('target_id' => $element);

    $library_items = entity_load('library_item', $items[$delta]->item_id );

    $element['library'] = array(
      '#type' => 'fieldset',
      '#title' => t('Item'),
      '#attributes' => array('class' => array('container-inline')),
    );

    $element['library']['item_id'] = array(
      '#type' => 'hidden',
      '#default_value' => isset($items[$delta]->item_id) ? $items[$delta]->item_id : $this->generateID($items),
    );

    $element['library']['barcode'] = array(
      '#type' => 'textfield',
      '#default_value' => 0,
      '#size' => 26,
      '#maxlength' => $this->getFieldSetting('max_length'),
      '#title' => t('Barcode'),
    );
    if ($this->generateBarcode()) {
      $element['library']['barcode']['#disabled'] = 'true';
      $element['library']['barcode']['#placeholder'] = t('Automatically generated');

    }
    $element['library']['in_circulation'] = array(
      '#type' => 'checkbox',
      '#default_value' => isset($items[$delta]->in_circulation) ? $items[$delta]->in_circulation : 0,
      '#title' => t('Reference only'),
    );
    $element['library']['library_status'] = array(
      '#type' => 'hidden',
      '#default_value' =>  isset($items[$delta]->status) ? $items[$delta]->status : 0,
    );
    $element['library']['notes'] = array(
        '#type' => 'textfield',
        '#default_value' => isset($items[$delta]->notes) ? $items[$delta]->notes : NULL,
        '#size' => '26',
        '#maxlength' => $this->getFieldSetting('max_length'),
        '#title' => t('Notes'),
    );
    $element['library']['created'] = array(
      '#type' => 'hidden',
      '#default_value' => isset($items[$delta]->created) ? $items[$delta]->created : time(),
      '#size' => $this->getSetting('size'),
    );

    return $element;
  }

  /**
   * Previously library was its own table with a unique key.
   *
   * This functions attempts to find the highest ever key used and use a larger
   * one. This shall prevent that the transactions are out-of-order when a
   * user rearranges the elements in the widget. The history would become
   * inconsistent if a user were to remove all items and add new ones
   * in the same node.
   *
   * @param $items
   * @return int
   */
  protected function generateID($items) {
    $highestElement = 0;
    foreach ($items as $item) {
      // Find the largest existing key.
      $segments = explode('-', $item->item_id);
      if (!empty($segments) && isset($segments[1])) {
        if ($segments[1] > $highestElement) {
          $highestElement = $segments[1];
        }
      }
    }
    // Legacy data is only auto-incremented, switch to a count here.
    if ($highestElement == 0) {
      $highestElement = count($items) - 1;
    }

    return $highestElement + 1;
  }

  protected function generateBarcode() {
    $entity = $this->fieldDefinition->getTargetEntityTypeId();
    $field = $this->fieldDefinition->getName();
    $limit = \Drupal::state()->get('library.barcode_base.' . $entity . '.' . $field);
    if ($limit) {
      return true;
    } else {
      return false;
    }
  }

}
